/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 1/01/05.
 *
 *  A test bed for a game. Can support multiple views and players.
 */

#ifndef CARGAME_H
#define CARGAME_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Player.h"
#include "ScView.h"
#include "ScTilemap.h"
#include "ScParticles.h"
#include "NeedleImage.h"
#include "PlayerBomb.h"
#include "Minimap.h"
#include <list>
#include <vector>
using namespace std;

// block
// Used in algorithm for improving collision detection with tiles by simplifying
// local tile geometry.

struct block
{
    block(int inRight, int inLeft, int inTop, int inBottom)
        : right(inRight), left(inLeft), top(inTop), bottom(inBottom) {}
    
    int right, left, top, bottom;
};

// ViewFollow
// A view and the player it is following

struct ViewFollow
{
	ViewFollow(ScView* inView, short inFollow) : view(inView), follow(inFollow) {}
	ScView* view;
	short follow;
};

// forwards declarations
class Wheel;
class WheelSprite;

// class CarGame

class CarGame
{
public:
			CarGame(ScTilemap* inTilemap, Player** inPlayers, int inNumPlayers, double inFuseTime, Tuple inStartLineA, Tuple inStartLineB);
			~CarGame();
	
	void	addView(ScPortRect& portRect, int playerFollow);
	void	doEvent(PlayerEvent* event, unsigned short playerNum = 0);
	void	step(double dt);
	
	void	renderEveryTexture();
	
	void	detonate(Player* player);
	
protected:
	ScTilemap&	tilemap;
	ScAnimation animation;
	Player**	players;
	const int	numPlayers;
	ScRect		gameRect;
	list<ViewFollow>	views;
	list<Player*>	playersInGame;
	ScParticles*	particles;
	NeedleImage*	needle;
	PlayerBomb*		bomb;
	Player*			playerWithBomb;
	double		bombFuseLength;
	double		runningTime;
	Minimap*	minimap;
	Tuple		startLineA, startLineB;
	
	void	computePhysics(double dt);
	void	drawViews();
	void	drawTime(double theTime);
	void	findTileContacts(ToyCar& car, list<Contact>& contacts);
	void	sewSeams(list<block>& blocks);
	void	resolveCollisions(list<Contact>& contacts);
	void	generateDustFromWheel(const Wheel& wheel, double dt, double rewind);
};

#endif
