/*
 *  Menu.cpp
 *  ToyCars
 *
 *  Created by Ruben Zilibowitz on 24/11/05.
 *  Copyright 2005 Ruben Zilibowitz. All rights reserved.
 *
 */

#include "Menu.h"
#include "GameConstants.h"

#if defined(__APPLE__) && defined(__MACH__)
#include <OpenGL/gl.h>	// Header File For The OpenGL32 Library
#else
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#endif

#include "SDL.h"
#include "SDL_image.h"

#include <cmath>

#include "ScException.h"
#include "glf.h"

inline int abs(int x)
{
    if (x < 0)
        return -x;
    
    return x;
}

Menu::Menu(char **inItems, int inNumItems, const char * inTitle)
    : items(inItems), numItems(inNumItems), title(inTitle)
{
}

int Menu::doMenu()
{
    int result;
	SDL_Event event;
    
	glColor4f(0.5f, 1.0f, 1.0f, 0.6f);
    glLoadIdentity();
	glOrtho(0, kScreenWidth, 0, kScreenHeight, -1, 1);
    
    if (title) {
        glPushMatrix();
        glTranslatef(kScreenWidth*0.5 - 128, kScreenHeight*0.5 + numItems*32 + 32, 0);
        glScalef(12, 12, 1);
        glfDrawSolidString(title);
        glPopMatrix();
    }
    
    glTranslatef(kScreenWidth*0.5 - 128, kScreenHeight*0.5 - numItems*32, 0);
    
	//int fontID = glfLoadFont("fonts/cricket1.glf");
	//if (fontID == GLF_ERROR)
	//	ScThrowErr("glfLoadFont returned GLF_ERROR");
	//glfSetCurrentFont(fontID);
    for (int i = numItems-1; i >= 0; i--)
    {
        glBegin(GL_LINE_LOOP);
        glVertex2f(0, 0);
        glVertex2f(256, 0);
        glVertex2f(256, 64);
        glVertex2f(0, 64);
        glEnd();
        glPushMatrix();
        glTranslatef(24, 32, 0);
        glScalef(12, 12, 1);
        glfDrawSolidString(items[i]);
        glPopMatrix();
        glTranslatef(0, 64, 0);
    }
	SDL_GL_SwapBuffers();
    //glfUnloadFont();
    
    while (1)
    {
		// get pending events
		while (SDL_PollEvent(&event))
		{
			//if (event.type == SDL_QUIT)
			//	running = false;
			
			if (event.type == SDL_MOUSEBUTTONDOWN)
            {
                if (abs(event.button.x - kScreenWidth/2) < 128)
                {
                    result = (int)floor((event.button.y - kScreenHeight*0.5 + numItems*32)/64.0);
                    if (result >= 0 && result < numItems)
                        return result;
                }
            }
        }
    }
}
