/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 2/01/05.
 */

#ifndef PLAYER_H
#define PLAYER_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ToyCar.h"

// PlayerEventType - describes the types of events a player can receive
typedef unsigned char PlayerEventType;

// Constants for different event types
const PlayerEventType kPlayerKeyUp = 0;
const PlayerEventType kPlayerKeyDown = 1;
const PlayerEventType kPlayerPaddleMove = 2;

// PlayerEvent - base class for player events
struct PlayerEvent
{
	PlayerEventType type;
};

// PlayerControl - describes the controls a player has
typedef unsigned char PlayerControl;

// PlayerEventPaddle - an event of a "paddle" changing value. Eg mouse moved or joystick moved.
struct PlayerEventPaddle : public PlayerEvent
{
	PlayerControl control;
	unsigned short value;
};

// PlayerKey - describes the keys a player has
typedef unsigned char PlayerKey;

// PlayerEventKey - an event of a key press or release
struct PlayerEventKey : public PlayerEvent
{
	PlayerKey key;
};

// the controls
enum
{
	kSteerWheel = 0,
	kThrottle,
	kBrake
};

// the keys
enum
{
	kSteerWheelLeft = 0,
	kSteerWheelRight,
	kThrottleMore,
	kThrottleLess,
	kBrakeMore,
	kBrakeLess,
	kNumKeys
};

// Player class

class Player
{
public:
			Player(ToyCar& inCar);
	
	void	doEvent(PlayerEvent* event);
	void	integrateKeys(double dt);
	
	ToyCar&	car;
	
	char	updateLaps(const Tuple& lineA, const Tuple& lineB, double curTime);
	short	getLaps()	{ return laps; }
	double	getBestLapTime()	{ return bestLapTime; }
	double	getCourseTime()		{ return courseTime; }
	
protected:
	void	keyUp(PlayerKey key) { keys[key] = false; }
	void	keyDown(PlayerKey key) { keys[key] = true; }
	void	paddleMove(PlayerControl which, unsigned short value);
	
	// key states
	bool	keys[kNumKeys];
	
	// lap counting variables
	char	lapFlag;
	short	laps;
	double	curLap, prevLap;
	
	// times
	double	bestLapTime;
	double	courseTime;
};

#endif
